/* Copyright (c) 2004, 2005 Dirk Jagdmann <doj@cubic.org>

This software is provided 'as-is', without any express or implied
warranty. In no event will the authors be held liable for any damages
arising from the use of this software.

Permission is granted to anyone to use this software for any purpose,
including commercial applications, and to alter it and redistribute it
freely, subject to the following restrictions:

    1. The origin of this software must not be misrepresented; you
       must not claim that you wrote the original software. If you use
       this software in a product, an acknowledgment in the product
       documentation would be appreciated but is not required.

    2. Altered source versions must be plainly marked as such, and
       must not be misrepresented as being the original software.

    3. This notice may not be removed or altered from any source
       distribution. */

// $Header: /code/doj/memorymap.hpp,v 1.7 2005/07/05 17:17:19 doj Exp $

#ifndef MEMORYMAP__HPP
#define MEMORYMAP__HPP

#include <stdint.h>
#include <string>

namespace doj
{

  /**
     map filename into memory. The size of the memory area can be
     retrieved with the memorymapsize(void*) function.

     @return a pointer to the memory mapped file. NULL on error
  */
  void* memorymap(const char *filename, const bool readonly=true);

  /**
     map filename into memory. The size of the memory area can be
     retrieved with the memorymapsize(void*) function.

     @return a pointer to the memory mapped file. NULL on error
  */
  inline void* memorymap(const std::string& filename, const bool readonly=true)
  {
    return memorymap(filename.c_str(), readonly);
  }

  /**
     unmap a previously memory mapped area.

     @param mem pointer to the beginning of the memory mapped file, as
     retrieved from the memorymap(...) function.

     @return 0 on success. -1 on failure.
  */
  int memoryunmap(void *mem);

  /**
     get the size of a memory mapped file.

     @param mem pointer to the beginning of the memory mapped file, as
     retrieved from the memorymap(...) function.

     @return the size of the memory area, which is identical to the
     filesize. 0 if the *mem does not resolve a valid start of a memory
     map.
  */
  uint64_t memorymapsize(void *mem);


  template <typename T>
  class memorymap_ptr
  {
    T *p;

  public:
    typedef T element_type;

    memorymap_ptr(const std::string& fn, const bool readonly=true) :
      p(reinterpret_cast<T*>(memorymap(fn.c_str(), readonly)))
    { }

    ~memorymap_ptr() { memoryunmap(p); }

    uint64_t size() const { return memorymapsize(p); }

    T& operator*() { return *p; }
    T* operator->() { return p; }
    T* get() { return p; }

    bool operator!() { return !p; }

  private:
    memorymap_ptr();
    memorymap_ptr(const memorymap_ptr&);
    memorymap_ptr& operator=(const memorymap_ptr&);
  };

}

#endif
